

available = [
    "included",
    "deposited",
    "released",
    "is provided",
    "are provided",
    "contained in",
    "available",
    "reproduce",
    "accessible",
    "can be accessed",
    "submitted",
    "can be downloaded",
    "reported in",
    "uploaded",
    "are public on"
    "provided"
    "archived"
]

was_available = [
    "was provided",
    "were provided",
    "was contained in",
    "were contained in",
    "was available",
    "were available",
    "was accessible",
    "were accessible",
    "deposited by",
    "were reproduced"
]

not_available = [
    "not included",
    "not deposited",
    "not released",
    "not provided",
    "not contained in",
    "not available",
    "not accessible",
    "not submitted"
]

field_specific_repo = {
    "GEO",
    "Deep Insight for Earth Science Data",
    "ESS-DIVE",
    "Gene Expression Omnibus",
    "European Nucleotide Archive",
    "National Center for Biotechnology Information",
    "European Molecular Biology Laboratory",
    "EMBL-EBI",
    "BioProject",
    "Sequence Read Archive",
    "SRA",
    "ENA",
    "MassIVE",
    "ProteomeXchange",
    "Proteome Exchange",
    "ProteomeExchange",
    "MetaboLights",
    "Array-Express",
    "ArrayExpress",
    "Array Express",
    "PRIDE",
    "DNA Data Bank of Japan",
    "DDBJ",
    "Genbank",
    "Protein Databank",
    "Protein Data Bank",
    "PDB",
    "Metagenomics Rapid Annotation using Subsystem Technology",
    "MG-RAST",
    "metabolights",
    "OpenAgrar",
    "Open Agrar",
    "Electron microscopy data bank",
    "emdb",
    "Cambridge Crystallographic Data Centre",
    "CCDC",
    "Treebase",
    "dbSNP",
    "dbGaP",
    "IntAct",
    "ClinVar",
    "European Variation Archive",
    "dbVar",
    "Mgnify",
    "NCBI Trace Archive",
    "NCBI Assembly",
    "UniProtKB",
    "Protein Circular Dichroism Data Bank",
    "PCDDB",
    "Worldwide Protein Data Bank",
    "wwPDB",
    "Structural Biology Data Grid",
    "NeuroMorpho",
    "G-Node",
    "Neuroimaging Informatics Tools and Resources Collaboratory",
    "NITRC",
    "EBRAINS",
    "GenomeRNAi",
    "Database of Interacting Proteins",
    "IntAct",
    "Japanese Genotype-phenotype Archive",
    "Biological General Repository for Interaction Datasets",
    "Genomic Expression Archive",
    "PeptideAtlas",
    "Environmental Data Initiative",
    "LTER Network Information System Data Portal",
    "Global Biodiversity Information Facility",
    "GBIF",
    "Integrated Taxonomic Information System",
    "ITIS",
    "Knowledge Network for Biocomplexity",
    "Morphobank",
    "The Network Data Exchange",
    "NDEx",
    "FlowRepository",
    "ImmPort",
    "Image Data Resource",
    "Eukaryotic Pathogen Database Resources",
    "EuPathDB",
    "Mouse Genome Informatics",
    "Rat Genome Database",
    "VectorBase",
    "Xenbase",
    "Zebrafish Model Organism Database",
    "ZFIN",
    "PhysioNet",
    "Research Domain Criteria Database",
    "RdoCdb",
    "Synapse",
    "UK Data Service",
    "caNanoLab",
    "ChEMBL",
    "STRENDA",
    "European Genome-phenome Archive",
    "European Genome phenome Archive",
    "Experimental Lakes Area",
    "IISD – ELA",
    "National Contaminants Information System",
    "NCIS",
    "Bedford Institute of Oceanography - Oceanographic Databases",
    "Institut océanographique de Bedford - Bases de données océanographiques",
    "Australian Urban Research Infrastructure Network",
    "AURIN",
    "Hakai Data",
    "Institute Data Systems",
    "Atlantic Canada Conservation Data Centre",
    "AC CDC",
    "Italian National Biodiversity Network",
    "Network Nazionale Biodiversità",
    "ForestPlots.net",
    "The Vegetation Plot Archive Project",
    "openLandscapes",
    "The Knowledge Collection for Landscape Science",
    "A vegetation data network for West Africa",
    "Canadian Environmental Sustainability Indicators",
    "CESI",
    "PPBio Data Repository",
    "Programa de Pesquisa em Biodiversidade Repositorio de Dados",
    "Environmental Information Data Centre",
    "EIDC",
    "National Biodiversity Information System of Mexico",
    "Sistema Nacional de Información sobre Biodiversidad de México",
    "Australian SuperSite Network Data Portal",
    "SupeSites Data Portal",
    "DEIMS-SDR",
    "Dynamic Ecological Information Management System",
    "Tree Atlas Project",
    "Marine Environmental Data Section",
    "Open Archive for Miscellaneous Data",
    "Biodiversity Exploratories Information System",
    "BExIS",
    "ICES data portal",
    "International Council for the Exploration of the Sea dataset collections",
    "Norwegian Marine Data Center",
    "Norsk marint datasenter",
    "Marine Microbial Database of India",
    "National Harvest Survey",
    "EarthWorks",
    "Environmental Data Initiative Repository",
    "EDI Data Portal",
    "The European Data Portal",
    "European data",
    "data.europa.eu",
    "Research Data Australia",
    "ARDC",
    "Santa Cruz Island Preserve Research Repository",
    "Dangermond Preserve Research Repository",
    "Draft Salmon Portal",
    "Arctic Data Portal",
    "NatCap",
    "California's Marine Protected Area Monitoring Program",
    "California MPA Monitoring Data Portal",
    "The Ocean Data Integration Initiative Testing Portal",
    "ODINI Testing Portal",
    "Ocean Biodiversity Information System",
    "OBIS",
    "accession number",
    "accession code",
    "accession numbers",
    "accession codes"
}

accession_nr = [
    "G(SE|SM|DS|PL)[[:digit:]]{2,}", #GEO
    "PRJ(E|D|N|EB|DB|NB)[:digit:]+",
    "SAM(E|D|N)[A-Z]?[:digit:]+",
    "[A-Z]{1}[:digit:]{4}", #GenBank
    "[A-Z]{2}[:digit:]{6}",
    "[A-Z]{3}[:digit:]{5}",
    "[A-Z]{4,6}[:digit:]{3,}",
    "GCA_[:digit:]{9}\\.[:digit:]+",
    "SR(P|R|X|S|Z)[[:digit:]]{3,}",
    "(E|P)-[A-Z]{4}-[:digit:]{1,}",
    "[:digit:]{1}[A-Z]{1}[[:alnum:]]{2}",
    "MTBLS[[:digit:]]{2,}",
    "10.17590",
    "10.5073",
    "10.25493",
    "10.6073",
    "10.15468",
    "10.5063",
    "[[:digit:]]{6}",
    "[A-Z]{2,3}_[:digit:]{5,}",
    "[A-Z]{2,3}-[:digit:]{4,}",
    "[A-Z]{2}[:digit:]{5}-[A-Z]{1}",
    "DIP:[:digit:]{3}",
    "FR-FCM-[[:alnum:]]{4}",
    "ICPSR [:digit:]{4}",
    "SN [:digit:]{4}"
]

repositories = [
    "figshare",
    "dryad",
    "zenodo",
    "dataverse",
    "DataverseNL",
    "osf",
    "open science framework",
    "mendeley data",
    "GIGADB",
    "GigaScience database",
    "OpenNeuro"
]

github = ["github", "github.com", "github.io", "githubusercontent.com", "githubusercontent.io", "githubpages.com", "githubpages.io", "githubusercontent", "githubpages"]

data = [
    "data", 
    "dataset",
    "datasets"]

all_data = [
    "all data",
    "raw data",
    "full data set",
    "full dataset",
    "complete data set",
    "complete dataset",
]

not_data = [
    "not all data",
    "no raw data",
    "no full data set",
    "no complete data set",
    "no complete dataset",
    "no full dataset"
]

source_code = [
    "source code",
    "analysis script",
    "github",
    "SAS script",
    "SPSS script",
    "R script",
    "R code",
    "python script",
    "python code",
    "matlab script",
    "matlab code"
]

supplement = [
    "supporting information",
    "supplement",
    "supplementary information",
    "supplementary material",
    "supplementary data"
]

file_formats = [
    "csv",
    "zip",
    "xls",
    "xlsx",
    "sav",
    "cif",
    "fasta"
]

upon_request = [
    "upon request",
    "on request",
    "upon reasonable request"
]
data_availability = [
    "Data sharing",
    "Data Availability Statement",
    "Data Availability",
    "Data deposition",
    "Deposited Data",
    "Data Archiving",
    "Availability of data and materials",
    "Availability of data",
    "Data Accessibility",
    "Data Accessibility Statement",
    "Accessibility of data"]

supplemental_table_name = [
    "supplementary table",
    "supplementary tables",
    "supplemental table",
    "supplemental tables",
    "table", "tables",
    "additional file",
    "file",
    "files"
]

supplemental_dataset = [
    "supplementary data [[:digit:]]{1,2}",
    "supplementary dataset [[:digit:]]{1,2}",
    "supplementary data set [[:digit:]]{1,2}",
    "supplemental data [[:digit:]]{1,2}",
    "supplemental dataset [[:digit:]]{1,2}",
    "supplemental data set [[:digit:]]{1,2}"
]



# original scripd binds dataset_name to dataset_numbering
dataset_name = [
    "data", "dataset", "datasets", "data set", "data sets"
]

dataset_number = ["S[[:digit:]]{1,2}"]

# TODO: Check this!
data_journal_dois = [
    "10.1038/s41597-019-", "10.3390/data", "10.1016/j.dib"
]
    
keywords = {
    "available": available,
    "was_available": was_available,
    "not_available": not_available,
    "field_specific_repo": field_specific_repo,
    "accession_nr": accession_nr,
    "repositories": repositories,
    "github": github,
    "data": data,
    "all_data": all_data,
    "not_data": not_data,
    "source_code": source_code,
    "supplement": supplement,
    "file_formats": file_formats,
    "upon_request": upon_request,
    "data_availability": data_availability,
    "supplemental_table_name": supplemental_table_name,
    "supplemental_dataset": supplemental_dataset,
    "dataset_name": dataset_name,
    "dataset_number": dataset_number,
    "data_journal_dois": data_journal_dois
}


# Web scraper looking for data on the web for a certain paper


from selenium import webdriver
from selenium.webdriver.common.by import By
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
import pandas as pd
from bs4 import BeautifulSoup
from EcoOpen.utils.keywords import keywords
import os
import requests

urls = [
    'https://link.springer.com/article/10.1007/s10886-017-0919-8',
    "https://link.springer.com/article/10.1007/s10886-018-0942-4",
    "https://doi.org/10.1073/pnas.1211733109",
    "http://dx.doi.org/10.3955/046.091.0105",
    "https://doi.org/10.1016/j.scitotenv.2013.10.121",
    "https://doi.org/10.1111/nph.14333"
    ]

filetypes = [
    "csv", "xlsx", "xls", "txt", "pdf", "zip",
    "tar.gz", "tar", "gz", "json", "xml", "docx",
    "doc", "ods", "odt", "pptx", "ppt", "png", "jpg",]

exclude = []
for i in filetypes:
    exclude.append(i.lower()+".")

def find_data(doi):
    url = "https://doi.org/" + doi
    driver = webdriver.Chrome()
    driver.maximize_window()
    driver.get(url)
    wait = WebDriverWait(driver, 10)
    real_url = driver.current_url
    print(real_url)
    domain = real_url.split('/')[2]
    soup = BeautifulSoup(driver.page_source, 'html.parser')
    supplementary_files = []
    closed_article_snippets = [
        "buy article",
        "access the full article",
        "purchase pdf",
    ]
    # check if the article is closed
    if any(snippet in str(soup).lower() for snippet in closed_article_snippets):
        print("You need to buy the article to access the data.")
    # specific for PNAS
    elif "suppl_file" in str(soup).lower():
        # find the link to the supplementary file
        links = soup.find_all('a')
        for link in links:
            try:
                if "suppl_file" in link.get('href'):
                    # print("There is a supplementary file")
                    supplementary_files.append(link.get('href'))
                    # print(link.get('href'))
            except TypeError:
                pass
    for i in keywords["repositories"]:
        links = soup.find_all('a')
        for link in links:
            try:
                if i in link.get('href'):
                    # print("There is a supplementary file")
                    supplementary_files.append(link.get('href'))
                    # print(link.get('href'))
            except TypeError:
                pass    
    return supplementary_files

def get_data_from_repo(link, output_dir="examples/data"):
    driver = webdriver.Chrome()
    driver.maximize_window()
    driver.get(link)
    wait = WebDriverWait(driver, 10)
    real_url = driver.current_url
    # print(real_url)
    domain = real_url.split('/')[2]
    soup = BeautifulSoup(driver.page_source, 'html.parser')
    files = []
    for i in filetypes:
        filename = ""
        links = soup.find_all('a')
        for link in links:
            try:
                if "."+i in link.get('href'):
                    # print("There is a file")
                    l = link.get('href')
                    if "http" not in l:
                        l = "https://"+domain + l
                    files.append(l)
                    # print(link.get('href'))
            except TypeError:
                pass
    # close the browser
    driver.close()
    # download the files
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    for i in files:
        # print(i)
        response = requests.get(i)
        response.raise_for_status()
        filename = i.split('/')[-1]
        if "?" in filename:
            filename = filename.split("?")[0]
        if filename != "":
            with open(os.path.join(output_dir, filename), 'wb') as f:
                f.write(response.content)
    return files

if __name__ == '__main__':
    doi = "10.1016/j.tpb.2012.08.002"
    doi = "10.3390/microorganisms10091765"
    data_links = find_data(doi)
    print(get_data_from_repo(data_links[0]))




import torch 
from transformers import AutoModelForCausalLM, AutoTokenizer, pipeline 

# Check if GPU is available
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
# check gpu memory ammount
vram = torch.cuda.get_device_properties(0).total_memory / 1024**3
device_ = "cuda"

if device.type != "cuda":
    raise ValueError("This function is not supported on CPU")
if vram < 12:
    device_ = "auto"
    print("""

ECOOPEN WARNING:
Not enough GPU memory to run the model, switching to combined mode, which will be slower. Please use other methods (non AI) to analyze the text.

""")
    

torch.random.manual_seed(0) 
model = AutoModelForCausalLM.from_pretrained( 
    "microsoft/Phi-3-mini-4k-instruct",  
    device_map=device_,  
    torch_dtype="auto",  
    trust_remote_code=True,  
)
tokenizer = AutoTokenizer.from_pretrained("microsoft/Phi-3-mini-4k-instruct") 

def get_inference(input_text):
    # print(input_text)
    answers = []
    questions = [
        f"Is the following text part of the scientific paper? {input_text}",
        f"Does the following text contains information about supplementary data, data availability or a link to the data? {input_text}"
    ]
    for q in questions:
        messages = [ 
            {"role": "system", "content": "You are analyzing a possible snippet of a scientific paper. You are answering with yes and no."}, 
            {"role": "user", "content": q},
        ]
        pipe = pipeline( 
            "text-generation", 
            model=model, 
            tokenizer=tokenizer, 
        )
        generation_args = { 
            "max_new_tokens": 500, 
            "return_full_text": False, 
            "temperature": 0.1, 
            "do_sample": False, 
        }
        output = pipe(messages, **generation_args ) 
        answers.append(output[0]['generated_text'])
        
    return answers

if __name__=="__main__":
    # create_model()
    print(get_inference("Data is available at the paper website. The link is www.sciom.hr/data/data.csv"))


from habanero import Crossref
import numpy as np
import re
from tqdm import tqdm
import pandas as pd
import itertools
from pathlib import Path
import sys
import requests
from bs4 import BeautifulSoup
import os
from urllib.parse import urlparse
import pathlib
from time import sleep
from tika import parser
from pprint import pprint
from tqdm import tqdm
from datetime import datetime

from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import cosine_similarity
from EcoOpen.inference import get_inference
from EcoOpen.utils.keywords import keywords
from EcoOpen.data_download import *


def find_papers(query="", doi=[], number_of_papers=100, start_year=2010, end_year=datetime.now().year+1, sort="published", order="desc"):
    cr = Crossref()
    if query != "":
        papers = cr.works(query=query, limit=number_of_papers, filter = {'from-pub-date': str(start_year), 'until-pub-date': str(end_year)}, sort=sort)
    elif doi != []:
        cr = Crossref()
        papers = cr.works(ids=doi)
    else:
        raise ValueError("Please provide a keyword, doi, title or author")
    
    dataframe = {
        "doi":[],
        "title":[],
        "authors":[],
        "published":[],
        "link":[],
    }   
    
    if type(papers) == dict:
    
        for paper in papers["message"]["items"]:
            # fill the dataframe. if the certain key is not present add empty string
            try:
                dataframe["doi"].append(paper["DOI"])
            except KeyError:
                dataframe["doi"].append("")
            try:
                dataframe["title"].append(paper["title"][0])
            except KeyError:
                dataframe["title"].append("")
            try:
                authors = ""
                for author in paper["author"]:
                    authors += author["given"] + " " + author["family"] + ", "
                authors = authors[:-2]
            
                dataframe["authors"].append(authors)
            except KeyError:
                dataframe["authors"].append("")
            try:
                dataframe["published"].append(paper["published"]["date-parts"][0])
            except KeyError:
                dataframe["published"].append("")
            try:
                dataframe["link"].append(paper["link"][0]["URL"])
            except KeyError:
                dataframe["link"].append("")
    elif type(papers) == list:
        for p in papers:
            # fill the dataframe. if the certain key is not present add empty string
            
            paper = p["message"]
            try:
                dataframe["doi"].append(paper["DOI"])
            except KeyError:
                dataframe["doi"].append("")
            try:
                dataframe["title"].append(paper["title"][0])
            except KeyError:
                dataframe["title"].append("")
            try:
                authors = ""
                for author in paper["author"]:
                    authors += author["given"] + " " + author["family"] + ", "
                authors = authors[:-2]
            
                dataframe["authors"].append(authors)
            except KeyError:
                dataframe["authors"].append("")
            try:
                dataframe["published"].append(paper["published"]["date-parts"][0])
            except KeyError:
                dataframe["published"].append("")
            try:
                dataframe["link"].append(paper["link"][0]["URL"])
            except KeyError:
                dataframe["link"].append
    
    return pd.DataFrame(dataframe)
    # return papers
    
def get_journal_info(journal_name):
    cr = Crossref()
    journal = cr.journals(query=journal_name)
    return journal

def get_paper_by_keyword(keyword):
    cr = Crossref()
    papers = cr.works(query=keyword)
    return papers

def get_papers(journal_list="available_journals.csv"):
    df = pd.read_csv(journal_list, index_col=0)
    df = df.dropna()
    df = df.reset_index()
    # print(df)

    for i in range(200, len(df)):
        # i = 101
        journal_title = df["Title"][i]
        expected_papers = df["Number of papers since 2010"][i]
        issn = df["ISSN"][i]
        issn = issn.replace("'", "")
        issn = issn.replace("[", "").replace("]", "")
        issn = issn.replace(" ", "").split(",")

        print(issn)

        noffsets=1
        if expected_papers/1000 >=1:
            noffsets = expected_papers//1000
        # jinfo = get_papers(issn)
        cr = Crossref()
        print(f"Searching for papers in {journal_title} since 01-01-2010")
        for iss in issn:
            jinfo = cr.journals(
                ids=issn,
                works=True,
                sort="published",
                order="asc",
                cursor="*",
                cursor_max=int(expected_papers),
                filter = {'from-pub-date': '2010-01-01'},
                progress_bar = True,
                limit=1000)

            print(type(jinfo))
            print(iss)

            if type(jinfo) == dict:
                items = jinfo["message"]["items"]
                if len(items) > 0:
                    break
            else:
                print(sum(len(z["message"]["items"]) for z in jinfo))

                items = [z["message"]["items"] for z in jinfo]
                items = list(itertools.chain.from_iterable(items))

        papers = {}

        keys = [
            "title",
            "published",
            "DOI",
            "type",
            #"abstract",
            "link",
            "is-referenced-by-count",
            "publisher",
            "author" # TODO: format
        ]
        for key in keys:
            papers[key] = []
        n=1
        for item in items:
            for key in keys:
                try:
                    value = item[key]
                    if key=="title":
                        value = value[0]
                    elif key in ("published", "issued"):
                        value = value["date-parts"][0]
                        if len(value) == 3:
                            value = f"{value[0]}-{value[1]}-{value[2]}"
                        elif len(value) == 2:
                            value = f"{value[0]}-{value[1]}"
                        else:
                            value = f"{value[0]}"
                    elif key == "link":
                        for v in value:
                            # print(v["URL"])
                            if "xml" not in v["URL"]:
                                value = v["URL"]
                            if "pdf" in v["URL"]:
                                value = v["URL"]
                                break
                            else:
                                value="no link"

                    if key == "author":
                        author_string=""
                        for a in value:
                            author_string+=a["given"] + " "
                            author_string+=a["family"]
                            if a["affiliation"] != []:
                                author_string+=" (" + str(a["affiliation"][0]["name"]) + "),"                    
                        value= author_string

                    papers[key].append(value)
                except KeyError:
                    # print(f"KeyError: {key}")
                    if key == "link":
                        key="URL"
                        try:
                            papers["link"].append(item[key])
                        except KeyError:
                            papers[key].append("")
                    else:
                        papers[key].append("")

            #print([item[k] for k in keys])

        jdf = pd.DataFrame(papers)
        jj = journal_title.replace(" ", "_")
        idx_ = i+1
        jdf.to_csv(f"data/papers/{idx_:03d}_{jj}.csv", sep=",", quoting=2)

    return "done!"


def custom_tokenizer(text):
    pattern = r"(?u)\b\w\w+\b[!]*"
    return re.findall(pattern, text) 

def gather_journal_info(list_of_journals, path_to_save=None):
    ISSNs = []
    titles = []
    query = []
    available = []
    dois = []
    number_of_papers_since_2010 = []
    journals = tqdm(list_of_journals, colour="green")

    for i in journals:
        jinfo = get_journal_info(i)

        if jinfo['message']["total-results"] > 0:

            # find item wit the most similar title
            # finding the right journal
            # in majority of cases the idx of the most similar journal title
            # will be 0, however sometimes API returns more popular journal
            # title as the first choice therefore an similarity checks between
            # query and result needs to be performed
            titles_ = [j["title"] for j in jinfo["message"]["items"]]
            qq = i
            vectorizer = TfidfVectorizer(
                tokenizer=custom_tokenizer, token_pattern=None)
            combined_list = titles_ + [qq]
            tfidf_matrix = vectorizer.fit_transform(combined_list)
            # print(tfidf_matrix)
            cosine_sim = cosine_similarity(
                tfidf_matrix[-1], tfidf_matrix[:-1]).flatten()

            idx = cosine_sim.argmax()
            # sanity check
            # if idx!=0:
            #     print(titles_)
            #     print(cosine_sim)
            ### idx points to the most similiar title
            
            available.append(1)
            ISSNs.append(jinfo['message']['items'][idx]['ISSN'])
            titles.append(jinfo['message']['items'][idx]['title'])
            
            npapers = 0
            dj = jinfo[
                "message"
                ]["items"][idx]["breakdowns"]["dois-by-issued-year"]
            for year in dj:
                if int(year[0]) >= 2010:
                    npapers += year[1]
            number_of_papers_since_2010.append(npapers)
            # print(
            #     i, "|",
            #     jinfo['message']["total-results"], "|",
            #     npapers, "|",
            #     titles[-1], "|",
            #     idx)
            res = jinfo['message']["total-results"]
            journals.set_description(
                f"{i} | {res} | {npapers} | {titles[-1]} | {idx}"
                )

        else:
            available.append(0)
            ISSNs.append("")
            titles.append("")
            number_of_papers_since_2010.append("")
            # print(
            #     i,
            #     jinfo['message']["total-results"]
            #     )
            journals.set_description(f"{i} | no result found")
        query.append(i)

    df = pd.DataFrame({
        "Journal query": query,
        "Title": titles,
        "ISSN": ISSNs,
        "Available": available,
        "Number of papers since 2010": number_of_papers_since_2010
    })
    # drop duplicate rows
    # df.drop_duplicates()
    if path_to_save:
        df.to_csv(path_to_save)
    return df

def download_paper(doi, output_dir):
    output_dir = Path(os.path.expanduser(output_dir))
    output_dir = output_dir.absolute()
    print(output_dir)
    cr = Crossref()
    doi = doi
    paper = cr.works(ids=doi)

    # print(paper)
    try:
        title = paper["message"]["title"][0]
    except KeyError:
        title = ""

    if "link" in paper["message"]:
        download_link = paper["message"]["link"][0]["URL"]
    elif "URL" in paper["message"]:
        download_link = paper["message"]["URL"]
        download_link = find_pdf(download_link)
    else:
        print("No download link found")
        download_link = ""

    print(download_link)
    if not os.path.exists(output_dir):
        os.system(f"mkdir {output_dir}")

    existing_papers = list(Path(f"{output_dir}").glob("**/*.pdf"))

    if "pdf" in download_link:
        os.system(f"wget --trust-server-names -O {output_dir}/{title.replace(' ', '_').replace('.', '_')}.pdf  {download_link}")
        new_path = f'{output_dir}/{title.replace(" ", "_").replace(".","_")}.pdf'
    else:
        os.system(f"python -m PyPaperBot --doi='{doi}' --dwn-dir='{output_dir}'")
        try:
            os.remove(f"{output_dir}/bibtex.bib")
            result = pd.read_csv(f"{output_dir}/result.csv")
            new_path = f'{output_dir}/{result["PDF Name"][0]}'
            # rename the downloaded file
            new_path = new_path.replace(" ", "_")
            os.rename(f"{output_dir}/{result['PDF Name'][0]}", new_path)
            os.remove(f"{output_dir}/result.csv")

        except FileNotFoundError:
            pass

    all_papers = list(Path(f"{output_dir}").glob("**/*.pdf"))
    if Path(new_path).exists():
        # get path to the downloaded file that was not present before
        print("Download successful")
        return new_path
    else:
        print("Download failed")
        return None

def find_pdf(url):
    # Requests URL and get response object
    response = requests.get(url)
    # print(response.url)

    # Parse text obtained
    soup = BeautifulSoup(response.text, 'html.parser')

    # Find all hyperlinks present on webpage
    links = soup.find_all('a')

    i = 0

    # From all links check for pdf link and
    # if present download file
    the_link = ""
    for link in links:
        if ('pdf' in link.get('href', [])):
            i += 1
            # print(link.get('href', []))
            the_link = link.get('href', [])
            break
    return "http://"+urlparse(response.url).netloc+the_link
        
def download_papers(dois, output_dir):
    download_results = {"doi":[], "path": [], "result":[]}
    for doi in dois:
        result = download_paper(doi, output_dir)
        if result != None:
            download_results["doi"].append(doi)
            download_results["path"].append(result)
            download_results["result"].append("Success")
        else:
            download_results["doi"].append(doi)
            download_results["path"].append("Download failed")
            download_results["result"].append("Failed")
            
    return pd.DataFrame(download_results)

def inference_cleanup(inference):
    new_inference = []
    for i in inference:
        i_=[i[0]]
        if "yes" in i[1][0].lower():
            i_.append("yes")
        elif "no" in i[1][0].lower():
            i_.append("no")

        if "yes" in i[1][1].lower():
            i_.append("yes")
        elif "no" in i[1][1].lower():
            i_.append("no")

        new_inference.append(i_)
    return new_inference

def find_dataKW(path):
    data_links = []
    return data_links

def find_dataAI(path):
    inference = AnalyzePDF_AI(path)
    indices = find_supplementary_data_sentence_AI(inference)
    data_links = []
    dataframe = {
        "data_links": [],
        "inference": []
    }
    if indices != []:
        for i in indices:
            # detect http links
            link = re.findall(r'(https?://\S+)', inference[i][0])
            if link != []:
                data_links.append(link)

        dataframe["data_links"] = data_links
        dataframe["inference"] = inference[i]
        
        
        return 
    
    else:
        print("No supplementary data found or data reference found in this paper")
        return None

def find_supplementary_data_sentence_AI(inference):
    indices = []
    for idx, i in enumerate(inference):
        if "yes" in i[2].lower():
            indices.append(idx)
    return indices

def ReadPDF(filepath):
    # parse PDF using 
    raw = parser.from_file(filepath)
    return raw["content"].replace("\n", "")

def ReadPDFs(filepaths):
    # if filepaths is a dataframe extract filepaths
    if type(filepaths) == pd.core.frame.DataFrame:
        filepaths = filepaths["path"].tolist()
    raws = []
    for file in filepaths:
        if type(file) == pathlib.PosixPath:
            file = str(file)
        raw = ReadPDF(file)
        raws.append(raw)
    return raws

def AnalyzePDF_AI(filepath):
    raw = ReadPDF(filepath)
    # split text into sentences
    sentences = re.split(r'(?<!\w\.\w.)(?<![A-Z][a-z]\.)(?<=\.|\?)\s', raw)
    # pprint(sentences)

    # split raw into paragraphs made from 5 sentences
    paragraphs = []
    for i in range(0, len(sentences), 3):
        paragraph = " ".join(sentences[i:i+5])
        paragraphs.append(paragraph)

    # get inference
    inference = []
    for i in tqdm(paragraphs):
        inf = get_inference(i)
        # print("Sentece", i)
        # print("Inference", inf)
        inference.append([i, inf])

    inference = inference_cleanup(inference)
    
    return inference


def FindOpenData(dois, output_dir):
    """Prototype function to find open data in scientific papers"""
    # download the papers
    download_results = download_papers(dois, "examples")
    # download the supplementary data
    supplementary_files = []
    for doi in dois:
        links = find_data(doi)
        print(links)
      
        
def find_das(sentences):
    """Find data availability sentences in the text"""
    das_keywords = keywords["data_availability"]
    das_sentences = [sentence for sentence in sentences if any(kw.lower() in sentence.lower() for kw in das_keywords)]
    return das_sentences

def find_keywords(sentences):
    """find keywords in sentences"""
    detected = []
    kw = keywords.copy()
    kw.pop("data_availability")
    for k in kw.keys():
        kk = kw[k]
        for sentence in sentences:
            if any(kw.lower() in sentence.lower() for kw in kk):
                detected.append(sentence)

    return detected

def find_dataKW(path):
    """Find data keywords in the text"""
    raw = ReadPDF(path)
    sentences = re.split(r'(?<!\w\.\w.)(?<![A-Z][a-z]\.)(?<=\.|\?)\s', raw)
    das_sentences = find_das(sentences)
    keyword_sentences = find_keywords(sentences)
    
    
    # detect links in the sentences
    data_links = []
    for sentence in keyword_sentences:
        link = re.findall(r'(https?://\S+)', sentence)
        if link != []:
            data_links.append(link)
    
    # clean the links
    dl = []
    for i in data_links:
        for j in i:
            # split merged links
            if len(j.split("http")) > 2:
                for k in j.split("http"):
                    if k != "":
                        dl.append("http"+k)
            else:
                dl.append(j)
    # clean special non standard url characters
    dl = [i.replace(")", "").replace("]", "").replace("}", "") for i in dl]
    dl = [i.replace("(", "").replace("[", "").replace("{", "") for i in dl]
    dl = [i.replace(";", "").replace(",", "") for i in dl]
    return das_sentences, dl
        


if __name__ == "__main__":
    # print(get_journal_info("Nature"))
    # print(get_paper_by_keyword("climate change"))
    # print(get_papers())
    # print(gather_journal_info(["Nature", "Science"]))
    # download_paper("10.3390/microorganisms10091765", "examples")
    # print(
        # download_papers(
        # ["10.1016/j.tpb.2012.08.002", "10.3390/microorganisms10091765"],
        # "examples")
    # )

    # example for reading PDF
    # raw = ReadPDF(
        # "examples/Stage_and_age_structured_Aedes_vexans_and_Culex_pipiens__Diptera__Culicidae__climate-dependent_matrix_population_model.pdf"
    # )
    # String of the content of the PDF
    # example_pdfs = list(pathlib.Path("examples").glob("*.pdf"))
    # raws = ReadPDFs(list(pathlib.Path("examples").glob("*.pdf")))   
    # print(raw)
    # example for analyzing PDF
    # inference = AnalyzePDF_AI(raws[1])
    # find_supplementary_data_sentence_AI(inference)
    # keywords = AnalyzePDF_keywords(raw)

    papers = find_papers("Hackenberger")
    
    
    
    papers




from setuptools import setup, find_packages

setup(
    name="EcoOpen",
    version="0.0.18",
    packages=find_packages(),
    install_requires=[
        "habanero",
        "numpy",
        "pandas",
        "scipy",
        "ollama",
        "beautifulsoup4",
        "scikit-learn",
        "pypaperbot",
        "ollama",
        "transformers",
        "datasets",
        "torch",
        "torchvision",
        "torchaudio",
        "accelerate",
        "selenium",
        "webdriver_manager",
        "tika"
    ],
    # entry_points={
    #     "console_scripts": [
    #         "EcoOpen=EcoOpen.core:main"
    #     ]
    # }
    author="Sciom d.o.o.",
    author_email="domagoj@sciom.hr",
    description="A tool for gathering information about data and data in scientific journals.",
    license="MIT",
    keywords="scientific journals, LLM, open access",
    url="https://sciom.hr"
)